/**
 * Common js code
 */

var nmgr = window.nmgr || {};

jQuery(function($) {

    /**
     * Define a common blockUI block setup for plugin
     *
     * @param {string | jQuery object} selector The element to be blocked
     */
    nmgr.block = function(selector) {
        if (!nmgr.is_blocked(selector)) {
            $(selector).addClass('is-blocked').block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#fff'
                },
                css: {
                    backgroundColor: 'transparent',
                    border: 'none'
                }
            });
        }
    };

    /**
     * Define a common blockUI unblock setup for plugin
     *
     * @param {string | jQuery object} selector The element to be unblocked
     */
    nmgr.unblock = function(selector) {
        $(selector).removeClass('is-blocked').unblock();
    };

    /**
     * Check whether an element is currently blocked with blockUI
     *
     * @param {string | jQuery object} selector The element to be checked
     * @returns {jQuery.length|Boolean} True if the element is blocked, false if not
     */
    nmgr.is_blocked = function(selector) {
        return $(selector).is('is-blocked') || $(selector).parents('is-blocked').length;
    };

    /**
     * Scroll to an element
     *
     * @param {string | jQuery object} selector The element to scroll to
     */
    nmgr.scroll_to = function(selector) {
        if ($(selector).length) {
            $('html, body').animate({
                scrollTop: ($(selector).offset().top - 100)
            }, 1000);
        }
    };

    /**
     * Go to a url
     * @param {string} url
     */
    nmgr.go_to = function(url) {
        if (-1 === url.indexOf('https://') || -1 === url.indexOf('http://')) {
            window.location = url;
        } else {
            window.location = decodeURI(url);
        }
    };

    /**
     * Remove woocommerce notices from the page
     * @returns {undefined}
     */
    nmgr.remove_notice = function() {
        $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
    };

    /**
     * Make a table responsive when it's width is above a certain size.
     *
     * The responsiveness in this case is for desktop display rather than mobile.
     * By default, the table should already be mobile responsive.
     *
     * @param {int} width Minimum width at which table would be responsive. Default 700px
     */
    nmgr.make_table_responsive = function() {
        var table = $('.nmgr-table');
        if (table.width() > 700) {
            table.addClass('responsive');
        } else {
            table.removeClass('responsive');
        }
    };

    nmgr.make_table_responsive();
    $(window).on('resize', nmgr.make_table_responsive);

    nmgr.tiptip = function() {
        $('.nmgr-tip').tipTip();

        $('svg.nmgr-tip').each(function() {
            var $element = $(this),
                title = $element.find('title');

            if (!title.length) {
                return;
            }

            $element.tipTip({
                content: function() {
                    if (title.length) {
                        if (title.attr('data-title')) {
                            title.text('');
                            return title.attr('data-title');
                        }
                    }
                }
            });
        });
    };

    nmgr.tiptip();


});


jQuery(function($) {

    if (typeof nmgr_account_params === 'undefined') {
        return false;
    }

    /*
     * Account tabs
     */

    var nmgrTab = function(element) {
        this.container = $(element);
        this.tabSelectedClass = "nmgr-tab-selected";
        this.tabHoverClass = "nmgr-tab-hover";
        this.tabSelector = this.container.find('.nmgr-tab');
        this.tabContentSelector = this.container.find('.nmgr-tab-content');
        this.tabContentVisibleClass = 'nmgr-visible';

        if (!this.container.length) {
            return;
        }

        this.showDefaultContent();

        this.tabSelector.on('mouseenter', {
            nmgrTab: this
        }, this.onTabMouseEnter);
        this.tabSelector.on('mouseleave', {
            nmgrTab: this
        }, this.onTabMouseLeave);
        this.tabSelector.on('click', {
            nmgrTab: this
        }, this.onTabClick);
        $(window).on('hashchange', {
            nmgrTab: this
        }, this.onHashChange);
    };

    nmgrTab.prototype.showDefaultContent = function() {
        if (this.getHash()) {
            this.showContent(this.getHash());
        } else {
            this.showContent($(this.tabSelector).first());
        }
    };

    nmgrTab.prototype.onHashChange = function(e) {
        var hash = e.data.nmgrTab.getHash();
        e.data.nmgrTab.showContent(hash);
        nmgr.scroll_to(hash);
    };

    nmgrTab.prototype.getHash = function() {
        var hash = window.location.hash;
        return (hash.toLowerCase().indexOf('nmgr-') >= 0) ? hash : false;
    };

    nmgrTab.prototype.onTabMouseEnter = function(e) {
        $(this).addClass(e.data.nmgrTab.tabHoverClass);
    };

    nmgrTab.prototype.onTabMouseLeave = function(e) {
        $(this).removeClass(e.data.nmgrTab.tabHoverClass);
    };

    nmgrTab.prototype.onTabClick = function(e) {
        e.preventDefault();
        e.data.nmgrTab.showContent($(this));
    };

    nmgrTab.prototype.showContent = function(tabElement) {
        if ('undefined' === $(tabElement) || !$(tabElement).is(this.tabSelector) || !$(tabElement).length) {
            tabElement = $(this.tabSelector).first();
        }

        var $link = $(tabElement).find('a').attr('href');

        $(this.tabSelector).removeClass(this.tabSelectedClass);
        $(this.tabContentSelector).removeClass(this.tabContentVisibleClass).hide();

        $(tabElement).addClass(this.tabSelectedClass);
        this.container.find($link).addClass(this.tabContentVisibleClass).show();

        /**
         * Trigger event when a tab's content is shown
         *
         * @param jQuery object|string $tab The tab element header clicked
         */
        $(document.body).trigger('nmgr_tab_shown', [tabElement]);

    };

    nmgr.account_tabs = new nmgrTab('#nmgr-account-tabs');

    $('.nmgr-go-to-tab').on('click', function(e) {
        var href = $(this).attr('href');
        if (href.toLowerCase().indexOf('#nmgr-tab') >= 0) {
            e.preventDefault();
            nmgr.scroll_to(href);
            nmgr.account_tabs.showContent(href);
        }
    });
    /**
     * Js events related to wishlist overview template
     */

    var nmgr_overview = {
        container: '#nmgr-overview',

        wishlist: {},

        /**
         * Flag to check if this template should be reloaded.
         * Used to prevent multiple reload events
         * @type Boolean
         */
        do_reload: false,

        init: function() {
            $(document.body)
                .on('click', '.nmgr-share-wishlist .nmgr-copy', this.copy_link)
                .on('nmgr_items_reloaded nmgr_profile_reloaded nmgr_shipping_reloaded', this.reload);
        },

        /**
         * Copy wishlist link to clipboard
         */
        copy_link: function(e) {
            e.preventDefault();
            var $temp = $("<input>");
            $("body").append($temp);
            $temp.val($('.nmgr-share-wishlist .link').text()).select();
            document.execCommand("copy");
            $('.nmgr-share-wishlist .nmgr-copy').text(nmgr_account_params.i18n_copied_text);
            $temp.remove();
        },

        reload: function(e, wishlist) {
            var this_wishlist_id = parseInt($(nmgr_overview.container).attr('data-wishlist-id')),
                wishlist_id = ('undefined' !== typeof wishlist) && wishlist.id ? wishlist.id : this_wishlist_id;

            if (nmgr_overview.do_reload || (0 !== this_wishlist_id && ('undefined' !== typeof wishlist) && wishlist.id && this_wishlist_id !== parseInt(wishlist.id))) {
                return;
            }

            nmgr_overview.do_reload = true;

            var data = {
                action: 'nmgr_load_overview',
                nmgr_global: nmgr_global_params.global,
                wishlist_id: parseInt(wishlist_id),
                security: nmgr_account_params.overview_nonce
            };

            nmgr.block(nmgr_overview.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                $(nmgr_overview.container).replaceWith(response);
                nmgr_overview.wishlist = wishlist;
                nmgr_overview.reloaded();
            });
        },

        reloaded: function() {
            setTimeout(function() {
                nmgr_overview.do_reload = false;
            }, 500);

            nmgr.tiptip();
            $(document.body).trigger('nmgr_overview_reloaded', this.wishlist);
        }
    };

    nmgr_overview.init();

    /**
     * Js events related to wishlist profile template
     */

    var nmgr_profile = {
        container: '#nmgr-profile',

        noticeTimer: '',

        wishlist: {},

        init: function() {
            this.init_datepicker();

            $(document.body)
                .on('submit', '.nmgr-profile-form', this.submit)
                .on('nmgr_profile_reloaded', this.update_tab_title);
        },

        init_datepicker: function() {
            return new Pikaday({
                field: document.getElementById('nmgr_event_date'),
                i18n: nmgr_global_params.i18n_pikaday
            });
        },

        reload: function(e, wishlist) {
            var this_wishlist_id = parseInt($(nmgr_profile.container).attr('data-wishlist-id')),
                wishlist_id = ('undefined' !== typeof wishlist) && wishlist.id ? wishlist.id : this_wishlist_id;

            if (0 !== this_wishlist_id && ('undefined' !== typeof wishlist) && wishlist.id && this_wishlist_id !== parseInt(wishlist.id)) {
                return;
            }

            var data = {
                action: 'nmgr_load_profile',
                wishlist_id: parseInt(wishlist_id),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_account_params.profile_nonce
            };

            nmgr.block(nmgr_profile.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                $(nmgr_profile.container).replaceWith(response);
                nmgr_profile.wishlist = wishlist;
                nmgr_profile.reloaded();
            });
        },

        reloaded: function() {
            this.init_datepicker();
            $(document.body).trigger('nmgr_profile_reloaded', this.wishlist);
        },

        showNotice: function(element) {
            $(element).addClass('nmgr-notice').prependTo($(nmgr_profile.container));
            clearTimeout(this.noticeTimer);
            this.noticeTimer = setTimeout(function() {
                $('#nmgr-profile .nmgr-notice').fadeOut(400, function() {
                    $(this).remove();
                });
            }, 7000);
        },

        submit: function(e) {
            e.preventDefault();

            var $form = $(this);

            var data = {
                data: $form.serialize(),
                action: 'nmgr_save_profile',
                wishlist_id: $(nmgr_profile.container).attr('data-wishlist-id'),
                nonce: $(nmgr_profile.container).attr('data-nonce'),
                nmgr_global: nmgr_global_params.global
            };

            nmgr.block($form);

            $.ajax({
                url: nmgr_global_params.ajax_url,
                data: data,
                type: 'POST',
                dataType: 'json',
                success: function(response) {
                    if (response.success && response.data) {

                        if (response.data.wishlist) {
                            nmgr_profile.wishlist = response.data.wishlist;
                        }

                        if (response.data.redirect) {
                            nmgr.go_to(response.data.redirect);
                        } else {
                            if (response.data.html) {
                                $(nmgr_profile.container).replaceWith($(response.data.html));
                            }

                            if (response.data.created) {
                                $(document.body).trigger('nmgr_wishlist_created', nmgr_profile.wishlist);
                            }

                            nmgr_profile.reloaded();
                        }
                    }

                    if (response.data && response.data.notice) {
                        nmgr_profile.showNotice(response.data.notice);
                        nmgr.scroll_to($('#nmgr-profile [role="alert"]'));
                    }

                    nmgr.unblock($form);

                }
            });
            return false;
        },

        update_tab_title: function(e, wishlist) {
            if (wishlist.id !== 0) {
                // tab id is hardcoded for now
                var icon = $('#nmgr-tab-profile').find('.nmgr-icon');
                if (icon.length && icon.attr('data-notice') === 'require-profile' && !icon.hasClass('nmgr-hide')) {
                    icon.addClass('nmgr-hide');
                }
            }
        }
    };

    nmgr_profile.init();

    /**
     * Js events related to wishlist items template
     */


    var nmgr_items = {
        container: '#nmgr-items',

        wishlist: {},

        init: function() {
            this.stupidtable.init();
            this.checkValidation();

            $(document.body)
                .on('change', '#nmgr-items input.quantity', this.quantity_changed)
                .on('click', '#nmgr-items a.edit-wishlist-item', this.edit_item)
                .on('click', '#nmgr-items a.delete-wishlist-item', this.delete_item)
                .on('click', '#nmgr-items button.save-action', this.save_items)
                .on('click', '#nmgr-items .nmgr-add-items-action', this.add_items_action)
                .on('change', '.nmgr-product-search', this.new_add_items_row)
                .on('click', '.nmgr-add-items .nmgr-add', this.add_items)
                .on('nmgr_wishlist_created nmgr_shipping_reloaded', this.reload)
                .on('nmgr_tab_shown', this.maybe_make_table_responsive);
        },

        reload: function(e, wishlist) {
            var this_wishlist_id = parseInt($(nmgr_items.container).attr('data-wishlist-id')),
                wishlist_id = ('undefined' !== typeof wishlist) && wishlist.id ? wishlist.id : this_wishlist_id;

            if (0 !== this_wishlist_id && ('undefined' !== typeof wishlist) && wishlist.id && this_wishlist_id !== parseInt(wishlist.id)) {
                return;
            }

            var data = {
                action: 'nmgr_load_items',
                wishlist_id: parseInt(wishlist_id),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_global_params.items_nonce
            };

            nmgr.block(nmgr_items.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                $(nmgr_items.container).replaceWith(response);
                nmgr_items.wishlist = wishlist;
                nmgr_items.reloaded();
            });
        },

        reloaded: function() {
            $(document.body).trigger('nmgr_items_reloaded', this.wishlist);
            this.stupidtable.init();
            this.checkValidation();
            nmgr.make_table_responsive();
        },

        checkValidation: function() {
            $('#nmgr-items input').on('invalid', function() {
                $(this).closest('tr').find('.edit').show();
                $(this).closest('tr').find('.view').hide();
                $('#nmgr-items button.save-action').attr('data-reload', true);
            });
        },

        quantity_changed: function() {
            var $row = $(this).closest('tr.item');
            var qty = $(this).val();
            var item_total = $('input.item_total', $row);
            var item_cost = $('.cost', $row).attr('data-sort-value');
            item_total.val(item_cost * qty);
        },

        edit_item: function() {
            $(this).closest('tr').find('.view').toggle();
            $(this).closest('tr').find('.edit').toggle();
            $('#nmgr-items button.save-action').attr('data-reload', true);
            return false;
        },

        delete_item: function() {
            var answer = window.confirm(nmgr_global_params.i18n_delete_item_text);
            if (answer) {

                nmgr.block(nmgr_items.container);

                var $item = $(this).closest('tr.item'),
                    data = {
                        wishlist_item_ids: $item.attr('data-wishlist_item_id'),
                        action: 'nmgr_delete_item',
                        wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                        nonce: $(nmgr_items.container).attr('data-nonce'),
                        nmgr_global: nmgr_global_params.global,
                        security: nmgr_global_params.items_nonce
                    };

                // Check if items have changed, if so pass them through so we can save them before adding a new item.
                if ('true' === $('#nmgr-items button.save-action').attr('data-reload')) {
                    data.items = $('.nmgr-items-table :input[name]').serialize();
                }

                $.post(nmgr_global_params.ajax_url, data, function(response) {
                    if (response.success) {
                        nmgr_items.wishlist = response.data.wishlist;
                        $(nmgr_items.container).replaceWith($(response.data.html));
                        nmgr_items.reloaded();
                    } else if (response.error) {
                        window.alert(response.data.notice);
                    }
                });
            }
            return false;
        },

        save_items: function() {
            var data = {
                items: $('.nmgr-items-table :input[name]').serialize(),
                action: 'nmgr_save_items',
                wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                nonce: $(nmgr_items.container).attr('data-nonce'),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_global_params.items_nonce
            };

            nmgr.block(nmgr_items.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                if (response.success) {
                    nmgr_items.wishlist = response.data.wishlist;
                    $(nmgr_items.container).replaceWith($(response.data.html));
                    nmgr_items.reloaded();
                }
                nmgr.unblock(nmgr_items.container);
            });
            return false;
        },

        add_items_action: function() {
            if ($(this).attr('data-url')) {
                nmgr.go_to($(this).attr('data-url'));
            } else if ($(this).attr('data-target')) {
                // Initialize bootstrap native modal
                new BSN.Modal($(this).attr('data-target')).show();

                // reset the modal content form
                nmgr_items.reset_add_items_form($(this).attr('data-target'));

                // initialize select2 on the select field in the modal
                nmgr_items.search_products();
            }
        },

        reset_add_items_form: function(element) {
            if ('undefined' !== typeof element) {
                $(element).find('tr.added-row').remove();
                $(element).find('form').trigger('reset').find(':input').each(function() {
                    if ($(this).is('.nmgr-product-search')) {
                        $(this).val('');
                    }
                });
            }
        },

        new_add_items_row: function() {
            if (!$(this).closest('tr').is(':last-child')) {
                return;
            }

            var item_table = $(this).closest('table'),
                item_table_body = item_table.find('tbody'),
                index = item_table_body.find('tr').length,
                row = item_table_body.data('row').replace(/\[0\]/g, '[' + index + ']');

            item_table_body.append('<tr class="added-row">' + row + '</tr>');

            nmgr_items.search_products();
        },

        add_items: function() {
            var table_body = $(this).closest('.nmgr-add-items').find('table tbody'),
                rows = table_body.find('tr'),
                add_items = [];

            $(rows).each(function() {
                var item_id = $(this).find(':input[name="item_id"]').val(),
                    item_qty = $(this).find(':input[name="item_qty"]').val(),
                    item_fav = $(this).find(':input[name="item_fav"]').val();

                add_items.push({
                    'id': item_id,
                    'qty': item_qty ? item_qty : 1,
                    'fav': item_fav
                });
            });

            var modal_id = '#' + $(this).closest('.modal').attr('id');
            // Initializing the modal if it is already open hides it. We don't need to explicitly call hide()
            new BSN.Modal(modal_id);

            nmgr.block(nmgr_items.container);

            var data = {
                action: 'nmgr_add_item',
                wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                security: nmgr_global_params.items_nonce,
                nmgr_global: nmgr_global_params.global,
                data: add_items
            };

            // Check if items have changed, if so pass them through so we can save them before adding a new item.
            if ('true' === $('#nmgr-items button.save-action').attr('data-reload')) {
                data.items = $('.nmgr-items-table :input[name]').serialize();
            }

            $.ajax({
                type: 'POST',
                url: nmgr_global_params.ajax_url,
                data: data,
                dataType: 'json',
                success: function(response) {
                    if (response.success) {
                        $(nmgr_items.container).replaceWith(response.data.html);
                        nmgr_items.reloaded();
                    } else {
                        nmgr.unblock(nmgr_items.container);
                        window.alert(response.data.error);
                    }
                }
            });
        },

        search_products: function() {
            $(':input.nmgr-product-search').each(function() {
                var select2_args = {
                    allowClear: $(this).data('allow_clear') ? true : false,
                    placeholder: $(this).data('placeholder'),
                    minimumInputLength: $(this).data('minimum_input_length') ? $(this).data('minimum_input_length') : '3',
                    escapeMarkup: function(m) {
                        return m;
                    },
                    ajax: {
                        url: nmgr_global_params.ajax_url,
                        dataType: 'json',
                        delay: 250,
                        data: function(params) {
                            return {
                                term: params.term,
                                action: 'nmgr_json_search_products',
                                security: nmgr_admin_params.search_products_nonce,
                                exclude: $(this).data('exclude'),
                                exclude_type: $(this).data('exclude_type'),
                                include: $(this).data('include'),
                                limit: $(this).data('limit'),
                                display_stock: $(this).data('display_stock'),
                            };
                        },
                        processResults: function(data) {
                            var terms = [];
                            if (data) {
                                $.each(data, function(id, text) {
                                    terms.push({
                                        id: id,
                                        text: text
                                    });
                                });
                            }
                            return {
                                results: terms
                            };
                        },
                        cache: true
                    }
                };

                $(this).select2(select2_args);

            });
        },

        stupidtable: {
            init: function() {
                $('.nmgr-items-table').stupidtable();
                $('.nmgr-items-table').on('aftertablesort', this.add_arrow);
                $('.nmgr-items-table th').on('mouseenter', this.show_default_arrow);
                $('.nmgr-items-table th').on('mouseleave', this.remove_default_arrow);
            },

            add_arrow: function(event, data) {
                var th = $(this).find('th');
                var arrow = data.direction === 'asc' ? '&uarr;' : '&darr;';
                var index = data.column;
                th.find('.nmgr-arrow').remove();
                th.eq(index).append('<span class="nmgr-arrow">' + arrow + '</span>');
            },

            show_default_arrow: function() {
                if ($(this).hasClass('sortable') && $(this).find('.nmgr-arrow').length === 0) {
                    $(this).append('<span class="nmgr-arrow hover">&uarr;</span>');
                }
            },

            remove_default_arrow: function() {
                $(this).find('.nmgr-arrow.hover').remove();
            }
        },

        maybe_make_table_responsive: function(e, tab) {
            if ($(tab).is('#nmgr-tab-items')) {
                nmgr.make_table_responsive();
            }
        }
    };
    nmgr_items.init();

    /**
     * Js events related to wishlist shipping template
     */

    var nmgr_shipping = {
        container: '#nmgr-shipping',

        noticeTimer: '',

        wishlist: {},

        init: function() {
            this.ship_to_account_address();

            $(document.body)
                .on('nmgr_wishlist_created', this.reload)
                .on('change', '#nmgr_ship_to_account_address', this.ship_to_account_address)
                .on('submit', '.nmgr-shipping-form', this.submit)
                .on('nmgr_shipping_reloaded', this.update_tab_title);
        },

        reload: function(e, wishlist) {
            var this_wishlist_id = parseInt($(nmgr_shipping.container).attr('data-wishlist-id')),
                wishlist_id = ('undefined' !== typeof wishlist) && wishlist.id ? wishlist.id : this_wishlist_id;

            if (0 !== this_wishlist_id && ('undefined' !== typeof wishlist) && wishlist.id && this_wishlist_id !== parseInt(wishlist.id)) {
                return;
            }

            var data = {
                action: 'nmgr_load_shipping',
                wishlist_id: parseInt(wishlist_id),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_account_params.shipping_nonce
            };

            nmgr.block(nmgr_shipping.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                $(nmgr_shipping.container).replaceWith(response);
                nmgr_shipping.wishlist = wishlist;
                nmgr_shipping.reloaded();
            });
        },

        reloaded: function() {
            $(document.body).trigger('nmgr_shipping_reloaded', this.wishlist);
            this.ship_to_account_address();
        },

        ship_to_account_address: function() {
            var $container = $(nmgr_shipping.container),
                $input = $container.find('#nmgr_ship_to_account_address');

            if ($input.is(':checked')) {
                if ($input.closest('label').attr('data-save')) {
                    if (window.confirm(nmgr_account_params.i18n_use_account_shipping_address_text)) {
                        $('.nmgr-shipping-form').submit();
                    } else {
                        $input.prop('checked', false);
                    }
                } else {
                    $container.find('.account-shipping-address').slideDown();
                    $container.find('.wishlist-shipping-address').hide();
                }
            } else {
                $container.find('.account-shipping-address').slideUp();
                $container.find('.wishlist-shipping-address').slideDown();
            }

            /**
             * initialize select2 on woocommerce shipping country and state fields
             * This is useful to maintain the right form state after ajax reload
             */
            $(document.body).trigger('country_to_state_changed');
            $container.find('#shipping_country').trigger('refresh');
        },

        showNotice: function(element) {
            $(element).addClass('nmgr-notice').prependTo($(nmgr_shipping.container));
            clearTimeout(this.noticeTimer);
            this.noticeTimer = setTimeout(function() {
                $('#nmgr-shipping .nmgr-notice').fadeOut(400, function() {
                    $(this).remove();
                });
            }, 7000);
        },

        submit: function(e) {
            e.preventDefault();

            var $form = $(this);

            var data = {
                data: $form.serialize(),
                action: 'nmgr_save_shipping',
                wishlist_id: $(nmgr_shipping.container).attr('data-wishlist-id'),
                nonce: $(nmgr_shipping.container).attr('data-nonce'),
                nmgr_global: nmgr_global_params.global
            };

            nmgr.block($form);

            $.ajax({
                url: nmgr_global_params.ajax_url,
                data: data,
                type: 'POST',
                dataType: 'json',
                success: function(response) {
                    // always update the data-save attribute of the ship_to_account_address checkbox after form submission
                    $(nmgr_shipping.container).find('#nmgr_ship_to_account_address').closest('label').attr('data-save', 1);

                    if (response.data.wishlist) {
                        nmgr_shipping.wishlist = response.data.wishlist;
                    }

                    if (response.success && response.data.html) {
                        $(nmgr_shipping.container).replaceWith($(response.data.html));
                    }

                    if (response.data.notice) {
                        nmgr_shipping.showNotice(response.data.notice);
                        nmgr.scroll_to($('#nmgr-shipping [role="alert"]'));
                    }

                    // Always reload shipping template as ship to account address field is always saved.
                    nmgr_shipping.reloaded();

                    nmgr.unblock($form);
                }
            });
            return false;
        },

        update_tab_title: function(e, wishlist) {
            if (nmgr_account_params.require_shipping_address && nmgr_global_params.global.is_account_tab) {
                // tab id is hardcoded for now
                var icon = $('#nmgr-tab-shipping').find('.nmgr-icon');

                if (!icon.length || icon.attr('data-notice') !== 'require-shipping-address') {
                    return;
                }

                if (wishlist.has_shipping_address && !icon.hasClass('nmgr-hide')) {
                    icon.addClass('nmgr-hide');
                } else if (!wishlist.has_shipping_address) {
                    icon.removeClass('nmgr-hide');
                }
            }
        }
    };

    nmgr_shipping.init();




});