/**
 * Handles adding a product to the wishlist
 * either normally or via ajax
 */

/**
 * Common js code
 */

var nmgr = window.nmgr || {};

jQuery(function($) {

    /**
     * Define a common blockUI block setup for plugin
     *
     * @param {string | jQuery object} selector The element to be blocked
     */
    nmgr.block = function(selector) {
        if (!nmgr.is_blocked(selector)) {
            $(selector).addClass('is-blocked').block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#fff'
                },
                css: {
                    backgroundColor: 'transparent',
                    border: 'none'
                }
            });
        }
    };

    /**
     * Define a common blockUI unblock setup for plugin
     *
     * @param {string | jQuery object} selector The element to be unblocked
     */
    nmgr.unblock = function(selector) {
        $(selector).removeClass('is-blocked').unblock();
    };

    /**
     * Check whether an element is currently blocked with blockUI
     *
     * @param {string | jQuery object} selector The element to be checked
     * @returns {jQuery.length|Boolean} True if the element is blocked, false if not
     */
    nmgr.is_blocked = function(selector) {
        return $(selector).is('is-blocked') || $(selector).parents('is-blocked').length;
    };

    /**
     * Scroll to an element
     *
     * @param {string | jQuery object} selector The element to scroll to
     */
    nmgr.scroll_to = function(selector) {
        if ($(selector).length) {
            $('html, body').animate({
                scrollTop: ($(selector).offset().top - 100)
            }, 1000);
        }
    };

    /**
     * Go to a url
     * @param {string} url
     */
    nmgr.go_to = function(url) {
        if (-1 === url.indexOf('https://') || -1 === url.indexOf('http://')) {
            window.location = url;
        } else {
            window.location = decodeURI(url);
        }
    };

    /**
     * Remove woocommerce notices from the page
     * @returns {undefined}
     */
    nmgr.remove_notice = function() {
        $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
    };

    /**
     * Make a table responsive when it's width is above a certain size.
     *
     * The responsiveness in this case is for desktop display rather than mobile.
     * By default, the table should already be mobile responsive.
     *
     * @param {int} width Minimum width at which table would be responsive. Default 700px
     */
    nmgr.make_table_responsive = function() {
        var table = $('.nmgr-table');
        if (table.width() > 700) {
            table.addClass('responsive');
        } else {
            table.removeClass('responsive');
        }
    };

    nmgr.make_table_responsive();
    $(window).on('resize', nmgr.make_table_responsive);

    nmgr.tiptip = function() {
        $('.nmgr-tip').tipTip();

        $('svg.nmgr-tip').each(function() {
            var $element = $(this),
                title = $element.find('title');

            if (!title.length) {
                return;
            }

            $element.tipTip({
                content: function() {
                    if (title.length) {
                        if (title.attr('data-title')) {
                            title.text('');
                            return title.attr('data-title');
                        }
                    }
                }
            });
        });
    };

    nmgr.tiptip();


});


jQuery(function($) {

    if (typeof nmgr_add_to_wishlist_params === 'undefined') {
        return false;
    }

    var params = nmgr_add_to_wishlist_params,
        selector = {
            wrapper: '.nmgr-add-to-wishlist-wrapper',
            form: '.nmgr-add-to-wishlist-form',
            dropdown_menu: '.nmgr-dropdown-menu',
            add_to_wishlist_btn: '.nmgr-add-to-wishlist-button'
        };

    var add_to_wishlist = {
        init: function() {
            this.enable_button_keyboard_navigation();

            $(document.body)
                .on('click', selector.add_to_wishlist_btn, this.add)
                .on('focus', selector.add_to_wishlist_btn, this.focus_form)
                .on('blur', selector.add_to_wishlist_btn, this.unfocus_form)
                .on('blur', '.nmgr-dropdown-menu input', this.unfocus_form)
                .on('mouseenter focus', selector.add_to_wishlist_btn, this.position_dropdown)
                .on('nmgr_added_to_wishlist', this.added_to_wishlist);
        },

        /**
         * Ensure the add to wishlist button can be accessible with keyboard navigation
         * This is generally necessary for custom buttons which may not have applied the default button attributes
         */
        enable_button_keyboard_navigation: function() {
            $(selector.add_to_wishlist_btn).attr('tabindex', 0);
        },

        position_dropdown: function() {
            var dropdown = $(this).closest(selector.form).find(selector.dropdown_menu);
            if (!dropdown.length || dropdown.attr('data-position')) {
                return;
            }

            var el = dropdown[0];
            var container;

            while ('Element' === typeof el && el.parentNode) {
                el = el.parentNode;
                if (getComputedStyle(el, null).overflow === 'hidden') {
                    container = el;
                    break;
                }
            }

            if (typeof container === 'undefined') {
                return;
            }

            var elemBottomPosition = dropdown.offset().top + dropdown.outerHeight();
            var containerBottomPosition = $(container).offset().top + $(container).outerHeight();

            if (elemBottomPosition <= containerBottomPosition) {
                return;
            }

            dropdown.addClass('top').attr('data-position', -dropdown.outerHeight()).css('top', -dropdown.outerHeight());
        },

        focus_form: function() {
            $(this).closest(selector.form).addClass('focus').attr('aria-expanded', true);
        },

        unfocus_form: function() {
            var form = $(this).closest(selector.form),
                dropdown = form.find(selector.dropdown_menu);

            setTimeout(function() {
                if (!dropdown.find($(document.activeElement)).length) {
                    form.removeClass('focus').attr('aria-expanded', false);
                }
            });
        },

        /**
         * Events to perform after adding product to a wishlist
         * - highlight wishlist containing product
         * - reset form
         *
         * @param {object} e Event
         * @param {object} response Ajax response from the add to wishlist action
         * @param {jQuery object} $thisbutton The form submit button
         */
        added_to_wishlist: function(e, response, $thisbutton) {
            var $form = $thisbutton.parent(selector.form);
            var $checkedInput = $form.find('input[name=nmgr_wid]:checked');

            if ($checkedInput.length) {
                $checkedInput.parent().find('.in-wishlist-icon').addClass('in-wishlist');
            }

            $form[0].reset();
        },

        /**
         * Show notice on the page
         *
         * @param {string} html The notice html.
         * @param {jQuery oject} form The form element containing the add to wishlist button.
         */
        show_notice: function(html, form) {
            $target = $('.woocommerce-notices-wrapper:first') || $('.nmgr-add-to-wishlist-button').closest('.woocommerce');

            if ($target.length) {
                $target.prepend(html);
                $('html, body').animate({
                    scrollTop: ($target.offset().top - 100)
                }, 1000);
            } else {
                form.closest(selector.wrapper).after(html);
            }
        },
        /**
         * Add a product to a wishlist
         *
         * @param {object} e Event
         */
        add: function(e) {
            e.preventDefault();
            var $form = $(this).parent(selector.form);

            // Remove any 'view wishlists' button present in the form wrapper
            $form.parent(selector.wrapper).find('.nmgr-view-wishlist-button').remove();

            // Deal with variation form
            if ($(this).is('.disabled')) {
                if ($(this).is('.wc-variation-is-unavailable')) {
                    alert(params.i18n_unavailable_text);
                } else if ($(this).is('.wc-variation-selection-needed')) {
                    alert(params.i18n_make_a_selection_text);
                }
                return;
            }

            // We have a dropdown that is active
            if ($form.find(selector.dropdown_menu).length && !$(this).hasClass('disabled')) {

                // Make sure a grouped product is selected, for grouped products
                if ($form.hasClass('product-type-grouped')) {
                    var quantityInputs = $form.find('input.qty');

                    if (quantityInputs.length) {
                        var hasValue = false;
                        for (var i = 0; i < quantityInputs.length; i++) {
                            if ($(quantityInputs[i]).val() > 0) {
                                hasValue = true;
                            }
                        }

                        if (!hasValue) {
                            $form.trigger('focus');
                            alert([params.i18n_select_quantity_text]);
                            return;
                        }
                    } else if ($form.find('input[name*=nmgr_qty]:not(".qty")').length && !$form.find('input[name*=nmgr_qty]:not(".qty"):checked').length) {
                        $form.trigger('focus');
                        alert([params.i18n_select_product_text]);
                        return;
                    }
                }

            }

            if ($form.is('.nmgr-ajax-add-to-wishlist:not(".redirect, .disabled")') &&
                $(this).is(':not(".disabled")')) {
                add_to_wishlist.ajax_action($(this), $form);
                return;
            }

            $form.submit();
        },
        /**
         * Add a product to a wishlist via ajax
         *
         * @param {jQuery object} $thisbutton The form submit button
         * @param {jQuery object} $form The html form
         */
        ajax_action: function($thisbutton, $form) {
            $thisbutton.removeClass('added');
            $thisbutton.addClass('loading');

            $form.find('.nmgr-animate').removeClass('nmgr-animation-scaleshrink');

            var data = {
                action: 'nmgr_add_to_wishlist',
                security: params.nonce
            };
            var formdata = $form.serializeArray();

            $(formdata).each(function(index, obj) {
                data[obj.name] = obj.value;
            });

            $(document.body).trigger('nmgr_adding_to_wishlist', [$thisbutton, $form, data]);

            $.post(params.ajax_url, data, function(response) {
                if (!response) {
                    return;
                }

                $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
                $thisbutton.removeClass('loading');

                if (response.error) {
                    add_to_wishlist.show_notice(response.notice, $form);
                    return;
                } else if (response.success) {

                    $form.addClass('product-in-wishlist');

                    $form.find('.nmgr-animate').addClass('nmgr-animation-scaleshrink');

                    if ($form.is('.single')) {
                        add_to_wishlist.show_notice(response.notice, $form);
                    }

                    if (response.notice &&
                        $(response.notice).find('.wc-forward').length &&
                        $form.is('.archive') &&
                        !$form.parent(selector.wrapper).find('.nmgr-view-wishlist-button').length) {
                        var $wishlist_link = $('.wc-forward', $(response.notice))
                            .removeClass('button')
                            .addClass('nmgr-view-wishlist-button')[0]
                            .outerHTML;
                        $form.after($wishlist_link);
                    }
                }

                /**
                 * Trigger event after adding to wishlist
                 *
                 * @param {object} response The response from the add to wishlist action
                 *  - This is an object containing the error, success, notice and result values
                 *  - relating to the product added to the wishlist.
                 *  -- The 'result' key in the object contains the product_id, variation_id,
                 *  -- and wishlist_id of the product added to the wishlist
                 *
                 * @param {jQuery object} $thisbutton The button that was clicked to add to wishlist
                 * @param {jQuery object} $form The form parent element of the button
                 */
                $(document.body).trigger('nmgr_added_to_wishlist', [response, $thisbutton, $form]);
            });
        }
    };
    /*
     * Add variable products to the wishlist
     *
     * (Used on single product page)
     * @todo convert to class
     */
    var add_to_wishlist_variable = {
        init: function() {
            $('.variations_form')
                .on('found_variation.wc-variation-form', this.found_variation)
                .on('show_variation', this.show_variation)
                .on('hide_variation', this.hide_variation);
        },

        /**
         * Get the wishlist form attached to the variation form
         *
         * @param {jQuery object} obj Variation form
         * @returns {object}
         */
        get_wishlist_form: function(obj) {
            var product_id = $(obj).data('product_id');
            return $(obj).closest('.product').find('[data-nmgr_product_id=' + product_id + ']');
        },

        found_variation: function(e, variation) {
            var variationFormValues = $(this).serialize(),
                $form = add_to_wishlist_variable.get_wishlist_form(this);
            $form.find('input[name="nmgr_vid"]').val(variation.variation_id).change();
            $form.find('input[name="nmgr_wc_form_values"]').val(variationFormValues).change();
            $form.find('input.qty').attr('min', variation.min_qty).attr('max', variation.max_qty);
        },

        show_variation: function(e, variation, purchasable) {
            var $form = add_to_wishlist_variable.get_wishlist_form(this),
                $add_to_wishlist_btn = $form.find(selector.add_to_wishlist_btn);

            if (purchasable) {
                $form.removeClass('disabled');
                $add_to_wishlist_btn.removeClass('disabled wc-variation-selection-needed');
            } else {
                $form.addClass('disabled');
                $add_to_wishlist_btn
                    .removeClass('wc-variation-selection-needed')
                    .addClass('disabled wc-variation-is-unavailable');
            }
        },

        hide_variation: function() {
            var $form = add_to_wishlist_variable.get_wishlist_form(this);
            $form.addClass('disabled');
            $form.find(selector.add_to_wishlist_btn).addClass('disabled wc-variation-selection-needed');
        }
    };

    add_to_wishlist.init();
    add_to_wishlist_variable.init();
});