/**
 * Common js code
 */

var nmgr = window.nmgr || {};

jQuery(function($) {

    /**
     * Define a common blockUI block setup for plugin
     *
     * @param {string | jQuery object} selector The element to be blocked
     */
    nmgr.block = function(selector) {
        if (!nmgr.is_blocked(selector)) {
            $(selector).addClass('is-blocked').block({
                message: null,
                overlayCSS: {
                    backgroundColor: '#fff'
                },
                css: {
                    backgroundColor: 'transparent',
                    border: 'none'
                }
            });
        }
    };

    /**
     * Define a common blockUI unblock setup for plugin
     *
     * @param {string | jQuery object} selector The element to be unblocked
     */
    nmgr.unblock = function(selector) {
        $(selector).removeClass('is-blocked').unblock();
    };

    /**
     * Check whether an element is currently blocked with blockUI
     *
     * @param {string | jQuery object} selector The element to be checked
     * @returns {jQuery.length|Boolean} True if the element is blocked, false if not
     */
    nmgr.is_blocked = function(selector) {
        return $(selector).is('is-blocked') || $(selector).parents('is-blocked').length;
    };

    /**
     * Scroll to an element
     *
     * @param {string | jQuery object} selector The element to scroll to
     */
    nmgr.scroll_to = function(selector) {
        if ($(selector).length) {
            $('html, body').animate({
                scrollTop: ($(selector).offset().top - 100)
            }, 1000);
        }
    };

    /**
     * Go to a url
     * @param {string} url
     */
    nmgr.go_to = function(url) {
        if (-1 === url.indexOf('https://') || -1 === url.indexOf('http://')) {
            window.location = url;
        } else {
            window.location = decodeURI(url);
        }
    };

    /**
     * Remove woocommerce notices from the page
     * @returns {undefined}
     */
    nmgr.remove_notice = function() {
        $('.woocommerce-error, .woocommerce-message, .woocommerce-info').remove();
    };

    /**
     * Make a table responsive when it's width is above a certain size.
     *
     * The responsiveness in this case is for desktop display rather than mobile.
     * By default, the table should already be mobile responsive.
     *
     * @param {int} width Minimum width at which table would be responsive. Default 700px
     */
    nmgr.make_table_responsive = function() {
        var table = $('.nmgr-table');
        if (table.width() > 700) {
            table.addClass('responsive');
        } else {
            table.removeClass('responsive');
        }
    };

    nmgr.make_table_responsive();
    $(window).on('resize', nmgr.make_table_responsive);

    nmgr.tiptip = function() {
        $('.nmgr-tip').tipTip();

        $('svg.nmgr-tip').each(function() {
            var $element = $(this),
                title = $element.find('title');

            if (!title.length) {
                return;
            }

            $element.tipTip({
                content: function() {
                    if (title.length) {
                        if (title.attr('data-title')) {
                            title.text('');
                            return title.attr('data-title');
                        }
                    }
                }
            });
        });
    };

    nmgr.tiptip();


});


jQuery(function($) {

    if (typeof nmgr_admin_params === 'undefined') {
        return false;
    }

    function init_datepicker() {
        return new Pikaday({
            field: document.getElementById('nmgr_event_date'),
            i18n: nmgr_global_params.i18n_pikaday
        });
    };
    init_datepicker();

    /**
     * Js events related to wishlist items template
     */


    var nmgr_items = {
        container: '#nmgr-items',

        wishlist: {},

        init: function() {
            this.stupidtable.init();
            this.checkValidation();

            $(document.body)
                .on('change', '#nmgr-items input.quantity', this.quantity_changed)
                .on('click', '#nmgr-items a.edit-wishlist-item', this.edit_item)
                .on('click', '#nmgr-items a.delete-wishlist-item', this.delete_item)
                .on('click', '#nmgr-items button.save-action', this.save_items)
                .on('click', '#nmgr-items .nmgr-add-items-action', this.add_items_action)
                .on('change', '.nmgr-product-search', this.new_add_items_row)
                .on('click', '.nmgr-add-items .nmgr-add', this.add_items)
                .on('nmgr_wishlist_created nmgr_shipping_reloaded', this.reload)
                .on('nmgr_tab_shown', this.maybe_make_table_responsive);
        },

        reload: function(e, wishlist) {
            var this_wishlist_id = parseInt($(nmgr_items.container).attr('data-wishlist-id')),
                wishlist_id = ('undefined' !== typeof wishlist) && wishlist.id ? wishlist.id : this_wishlist_id;

            if (0 !== this_wishlist_id && ('undefined' !== typeof wishlist) && wishlist.id && this_wishlist_id !== parseInt(wishlist.id)) {
                return;
            }

            var data = {
                action: 'nmgr_load_items',
                wishlist_id: parseInt(wishlist_id),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_global_params.items_nonce
            };

            nmgr.block(nmgr_items.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                $(nmgr_items.container).replaceWith(response);
                nmgr_items.wishlist = wishlist;
                nmgr_items.reloaded();
            });
        },

        reloaded: function() {
            $(document.body).trigger('nmgr_items_reloaded', this.wishlist);
            this.stupidtable.init();
            this.checkValidation();
            nmgr.make_table_responsive();
        },

        checkValidation: function() {
            $('#nmgr-items input').on('invalid', function() {
                $(this).closest('tr').find('.edit').show();
                $(this).closest('tr').find('.view').hide();
                $('#nmgr-items button.save-action').attr('data-reload', true);
            });
        },

        quantity_changed: function() {
            var $row = $(this).closest('tr.item');
            var qty = $(this).val();
            var item_total = $('input.item_total', $row);
            var item_cost = $('.cost', $row).attr('data-sort-value');
            item_total.val(item_cost * qty);
        },

        edit_item: function() {
            $(this).closest('tr').find('.view').toggle();
            $(this).closest('tr').find('.edit').toggle();
            $('#nmgr-items button.save-action').attr('data-reload', true);
            return false;
        },

        delete_item: function() {
            var answer = window.confirm(nmgr_global_params.i18n_delete_item_text);
            if (answer) {

                nmgr.block(nmgr_items.container);

                var $item = $(this).closest('tr.item'),
                    data = {
                        wishlist_item_ids: $item.attr('data-wishlist_item_id'),
                        action: 'nmgr_delete_item',
                        wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                        nonce: $(nmgr_items.container).attr('data-nonce'),
                        nmgr_global: nmgr_global_params.global,
                        security: nmgr_global_params.items_nonce
                    };

                // Check if items have changed, if so pass them through so we can save them before adding a new item.
                if ('true' === $('#nmgr-items button.save-action').attr('data-reload')) {
                    data.items = $('.nmgr-items-table :input[name]').serialize();
                }

                $.post(nmgr_global_params.ajax_url, data, function(response) {
                    if (response.success) {
                        nmgr_items.wishlist = response.data.wishlist;
                        $(nmgr_items.container).replaceWith($(response.data.html));
                        nmgr_items.reloaded();
                    } else if (response.error) {
                        window.alert(response.data.notice);
                    }
                });
            }
            return false;
        },

        save_items: function() {
            var data = {
                items: $('.nmgr-items-table :input[name]').serialize(),
                action: 'nmgr_save_items',
                wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                nonce: $(nmgr_items.container).attr('data-nonce'),
                nmgr_global: nmgr_global_params.global,
                security: nmgr_global_params.items_nonce
            };

            nmgr.block(nmgr_items.container);

            $.post(nmgr_global_params.ajax_url, data, function(response) {
                if (response.success) {
                    nmgr_items.wishlist = response.data.wishlist;
                    $(nmgr_items.container).replaceWith($(response.data.html));
                    nmgr_items.reloaded();
                }
                nmgr.unblock(nmgr_items.container);
            });
            return false;
        },

        add_items_action: function() {
            if ($(this).attr('data-url')) {
                nmgr.go_to($(this).attr('data-url'));
            } else if ($(this).attr('data-target')) {
                // Initialize bootstrap native modal
                new BSN.Modal($(this).attr('data-target')).show();

                // reset the modal content form
                nmgr_items.reset_add_items_form($(this).attr('data-target'));

                // initialize select2 on the select field in the modal
                nmgr_items.search_products();
            }
        },

        reset_add_items_form: function(element) {
            if ('undefined' !== typeof element) {
                $(element).find('tr.added-row').remove();
                $(element).find('form').trigger('reset').find(':input').each(function() {
                    if ($(this).is('.nmgr-product-search')) {
                        $(this).val('');
                    }
                });
            }
        },

        new_add_items_row: function() {
            if (!$(this).closest('tr').is(':last-child')) {
                return;
            }

            var item_table = $(this).closest('table'),
                item_table_body = item_table.find('tbody'),
                index = item_table_body.find('tr').length,
                row = item_table_body.data('row').replace(/\[0\]/g, '[' + index + ']');

            item_table_body.append('<tr class="added-row">' + row + '</tr>');

            nmgr_items.search_products();
        },

        add_items: function() {
            var table_body = $(this).closest('.nmgr-add-items').find('table tbody'),
                rows = table_body.find('tr'),
                add_items = [];

            $(rows).each(function() {
                var item_id = $(this).find(':input[name="item_id"]').val(),
                    item_qty = $(this).find(':input[name="item_qty"]').val(),
                    item_fav = $(this).find(':input[name="item_fav"]').val();

                add_items.push({
                    'id': item_id,
                    'qty': item_qty ? item_qty : 1,
                    'fav': item_fav
                });
            });

            var modal_id = '#' + $(this).closest('.modal').attr('id');
            // Initializing the modal if it is already open hides it. We don't need to explicitly call hide()
            new BSN.Modal(modal_id);

            nmgr.block(nmgr_items.container);

            var data = {
                action: 'nmgr_add_item',
                wishlist_id: $(nmgr_items.container).attr('data-wishlist-id'),
                security: nmgr_global_params.items_nonce,
                nmgr_global: nmgr_global_params.global,
                data: add_items
            };

            // Check if items have changed, if so pass them through so we can save them before adding a new item.
            if ('true' === $('#nmgr-items button.save-action').attr('data-reload')) {
                data.items = $('.nmgr-items-table :input[name]').serialize();
            }

            $.ajax({
                type: 'POST',
                url: nmgr_global_params.ajax_url,
                data: data,
                dataType: 'json',
                success: function(response) {
                    if (response.success) {
                        $(nmgr_items.container).replaceWith(response.data.html);
                        nmgr_items.reloaded();
                    } else {
                        nmgr.unblock(nmgr_items.container);
                        window.alert(response.data.error);
                    }
                }
            });
        },

        search_products: function() {
            $(':input.nmgr-product-search').each(function() {
                var select2_args = {
                    allowClear: $(this).data('allow_clear') ? true : false,
                    placeholder: $(this).data('placeholder'),
                    minimumInputLength: $(this).data('minimum_input_length') ? $(this).data('minimum_input_length') : '3',
                    escapeMarkup: function(m) {
                        return m;
                    },
                    ajax: {
                        url: nmgr_global_params.ajax_url,
                        dataType: 'json',
                        delay: 250,
                        data: function(params) {
                            return {
                                term: params.term,
                                action: 'nmgr_json_search_products',
                                security: nmgr_admin_params.search_products_nonce,
                                exclude: $(this).data('exclude'),
                                exclude_type: $(this).data('exclude_type'),
                                include: $(this).data('include'),
                                limit: $(this).data('limit'),
                                display_stock: $(this).data('display_stock'),
                            };
                        },
                        processResults: function(data) {
                            var terms = [];
                            if (data) {
                                $.each(data, function(id, text) {
                                    terms.push({
                                        id: id,
                                        text: text
                                    });
                                });
                            }
                            return {
                                results: terms
                            };
                        },
                        cache: true
                    }
                };

                $(this).select2(select2_args);

            });
        },

        stupidtable: {
            init: function() {
                $('.nmgr-items-table').stupidtable();
                $('.nmgr-items-table').on('aftertablesort', this.add_arrow);
                $('.nmgr-items-table th').on('mouseenter', this.show_default_arrow);
                $('.nmgr-items-table th').on('mouseleave', this.remove_default_arrow);
            },

            add_arrow: function(event, data) {
                var th = $(this).find('th');
                var arrow = data.direction === 'asc' ? '&uarr;' : '&darr;';
                var index = data.column;
                th.find('.nmgr-arrow').remove();
                th.eq(index).append('<span class="nmgr-arrow">' + arrow + '</span>');
            },

            show_default_arrow: function() {
                if ($(this).hasClass('sortable') && $(this).find('.nmgr-arrow').length === 0) {
                    $(this).append('<span class="nmgr-arrow hover">&uarr;</span>');
                }
            },

            remove_default_arrow: function() {
                $(this).find('.nmgr-arrow.hover').remove();
            }
        },

        maybe_make_table_responsive: function(e, tab) {
            if ($(tab).is('#nmgr-tab-items')) {
                nmgr.make_table_responsive();
            }
        }
    };
    nmgr_items.init();

    var ajax_user_search = {
        init: function() {
            $(':input.nmgr-user-search').each(function() {
                var select2_args = {
                    allowClear: $(this).data('allow_clear') ? true : false,
                    placeholder: $(this).data('placeholder'),
                    minimumInputLength: $(this).data('minimum_input_length') ? $(this).data('minimum_input_length') : '1',
                    escapeMarkup: function(m) {
                        return m;
                    },
                    ajax: {
                        url: nmgr_global_params.ajax_url,
                        dataType: 'json',
                        delay: 250,
                        data: function(params) {
                            return {
                                term: params.term,
                                action: 'nmgr_json_search_users',
                                security: nmgr_admin_params.search_users_nonce,
                                exclude: $(this).data('exclude')
                            };
                        },
                        processResults: function(data) {
                            var terms = [];
                            if (data) {
                                $.each(data, function(id, text) {
                                    terms.push({
                                        id: id,
                                        text: text
                                    });
                                });
                            }
                            return {
                                results: terms
                            };
                        },
                        cache: true
                    }
                };

                $(this).select2(select2_args);

            });
        }
    };

    ajax_user_search.init();
    var ship_to_account_address = {

        selector: {
            ship_to: '#nmgr_ship_to_account_address',
            user_id: '#nmgr_user_id'
        },

        init: function() {
            if (typeof nmgr_admin_params === 'undefined') {
                return false;
            }

            this.toggle_fieldsets();

            $(this.selector.ship_to).on('change', this.maybe_toggle_fieldsets);
        },

        maybe_toggle_fieldsets: function() {
            if ($(this).is(':checked')) {
                if (!$(ship_to_account_address.selector.user_id).val()) {
                    alert(nmgr_admin_params.i18n_select_user);
                    $(this).prop('checked', false);
                    return false;
                } else {
                    if (!window.confirm(nmgr_admin_params.i18n_use_account_shipping_address_text)) {
                        $(this).prop('checked', false);
                        return false;
                    }
                }
            }
            ship_to_account_address.toggle_fieldsets();
        },

        toggle_fieldsets: function() {
            if ($(this.selector.ship_to).is(':checked')) {
                $('.account-shipping-address').slideDown();
                $('.wishlist-shipping-address').hide();
            } else {
                $('.account-shipping-address').slideUp();
                $('.wishlist-shipping-address').slideDown();
            }
        }
    };

    ship_to_account_address.init();

    /**
     * Use selectWoo to select the shipping country and state the woocommerce way in admin
     * This code is simply a modified version of the same code woocommerce uses to select shipping
     * country and state in the order screen. Modified from meta-boxes-order.js in woocommerce plugin.
     */
    var shipping_country_state = {
        states: null,

        init: function() {
            if (!(typeof nmgr_admin_params === 'undefined' || typeof nmgr_admin_params.countries === 'undefined')) {
                this.states = $.parseJSON(nmgr_admin_params.countries.replace(/&quot;/g, '"'));
            }

            $('.js_field-country').selectWoo().change(this.change_country);
            $('.js_field-country').trigger('change', [true]);
            $(document.body).on('change', 'select.js_field-state', this.change_state);
        },

        change_country: function(e, stickValue) {
            // Check for stickValue before using it
            if (typeof stickValue === 'undefined') {
                stickValue = false;
            }

            // Prevent if we don't have the metabox data
            if (shipping_country_state.states === null) {
                return;
            }

            var $this = $(this),
                country = $this.val(),
                $state = $this.parents('div.nmgr-shipping-fields').find(':input.js_field-state'),
                $parent = $state.parent(),
                input_name = $state.attr('name'),
                input_id = $state.attr('id'),
                value = $this.data('woocommerce.stickState-' + country) ? $this.data('woocommerce.stickState-' + country) : $state.val(),
                placeholder = $state.attr('placeholder'),
                $newstate;

            if (stickValue) {
                $this.data('woocommerce.stickState-' + country, value);
            }

            // Remove the previous DOM element
            $parent.show().find('.select2-container').remove();

            if (!$.isEmptyObject(shipping_country_state.states[country])) {
                var state = shipping_country_state.states[country],
                    $defaultOption = $('<option value=""></option>')
                    .text(nmgr_admin_params.i18n_select_state_text);

                $newstate = $('<select></select>')
                    .prop('id', input_id)
                    .prop('name', input_name)
                    .prop('placeholder', placeholder)
                    .addClass('js_field-state select short')
                    .append($defaultOption);

                $.each(state, function(index) {
                    var $option = $('<option></option>')
                        .prop('value', index)
                        .text(state[index]);
                    $newstate.append($option);
                });

                $newstate.val(value);

                $state.replaceWith($newstate);

                $newstate.show().selectWoo().hide().change();
            } else {
                $newstate = $('<input type="text" />')
                    .prop('id', input_id)
                    .prop('name', input_name)
                    .prop('placeholder', placeholder)
                    .addClass('js_field-state')
                    .val('');
                $state.replaceWith($newstate);
            }
        },

        change_state: function() {
            // Here we will find if state value on a select has changed and stick it to the country data
            var $this = $(this),
                state = $this.val(),
                $country = $this.parents('div.nmgr-shipping-fields').find(':input.js_field-country'),
                country = $country.val();

            $country.data('woocommerce.stickState-' + country, state);
        }
    };

    shipping_country_state.init();

});