<?php

defined( 'ABSPATH' ) || exit;

/**
 * Handles all plugin ajax events
 */
class NMGR_Ajax {

	public static function run() {
		self::ajax_events();
		add_action( 'admin_init', array( __CLASS__, 'set_global_variable' ), -1 );
		add_filter( 'nmgr_get_wishlist_data', array( __CLASS__, 'get_wishlist_data' ), 10, 2 );
	}

	/**
	 * Make the plugin global variable available for all ajax requests
	 *
	 * json_decode is used here for situations where JSON.stringify is used in
	 * the ajax script to post the global nmgr variable  (like when using the formData object)
	 */
	public static function set_global_variable() {
		if ( is_ajax() ) {
			if ( isset( $_REQUEST[ 'nmgr_global' ] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
				$nmgr = $_REQUEST[ 'nmgr_global' ]; // phpcs:ignore

				if ( is_string( $nmgr ) ) {
					$nmgr = ( array ) json_decode( stripslashes( $nmgr ) );
				}

				$GLOBALS[ 'nmgr' ] = ( object ) wc_clean( $nmgr );
			}
		}
	}

	/**
	 * Add extra properties to wishlist data for ajax requests
	 *
	 * @param array $data Wishlist data
	 * @param NMGR_Wishlist $wishlist
	 * @return array
	 */
	public static function get_wishlist_data( $data, $wishlist ) {
		if ( !is_ajax() ) {
			return $data;
		}

		$data = array_merge( $data, array(
			'has_shipping_address' => $wishlist->has_shipping_address(),
			) );

		return $data;
	}

	public static function ajax_events() {
		$ajax_events = array(
			'save_profile',
			'save_items',
			'save_shipping',
			'add_item',
			'add_to_wishlist',
			'delete_item',
			'json_search_products',
			'json_search_users',
			'load_overview',
			'load_profile',
			'load_items',
			'load_shipping',
		);

		foreach ( $ajax_events as $event ) {
			if ( method_exists( __CLASS__, $event ) ) {
				add_action( 'wp_ajax_nmgr_' . $event, array( __CLASS__, $event ) );
			}
		}

		$ajax_nopriv_events = array(
			'add_to_cart',
			'get_cart_fragments',
		);

		foreach ( $ajax_nopriv_events as $event ) {
			if ( method_exists( __CLASS__, $event ) ) {
				add_action( 'wp_ajax_nmgr_' . $event, array( __CLASS__, $event ) );
				add_action( 'wp_ajax_nopriv_nmgr_' . $event, array( __CLASS__, $event ) );
			}
		}

		/**
		 * Remove woocommerce's add to cart action when running our own so that the add to cart action
		 * runs only once for us
		 */
		if ( is_ajax() && isset( $_REQUEST[ 'action' ] ) && ('nmgr_add_to_cart' === $_REQUEST[ 'action' ]) ) { // phpcs:ignore WordPress.Security.NonceVerification
			remove_action( 'wp_loaded', array( 'WC_Form_Handler', 'add_to_cart_action' ), 20 );
		}
	}

	public static function add_to_wishlist() {
		check_ajax_referer( 'nmgr-add-to-wishlist', 'security' );

		$result = NMGR_Wordpress::add_to_wishlist_action();

		// We are expecting notices from the add to wishlist action, so get them
		$error = wc_notice_count( 'error' ) ? true : false;
		$success = wc_notice_count( 'success' ) ? true : false;

		// This clears the notices after printing so we get it last
		$notice = wc_print_notices( true );

		wp_send_json( array(
			'notice' => $notice,
			'error' => $error,
			'success' => $success,
			'result' => $result,
		) );
	}

	public static function add_to_cart() {
		// phpcs:disable WordPress.Security.NonceVerification
		if ( !isset( $_POST[ 'add-to-cart' ] ) ) {
			wp_die( -1 );
		}

		$product_id = absint( $_POST[ 'add-to-cart' ] );
		$quantity = empty( $_POST[ 'quantity' ] ) ? 1 : wc_stock_amount( wp_unslash( $_POST[ 'quantity' ] ) );
		$variation_id = isset( $_POST[ 'variation_id' ] ) ? absint( $_POST[ 'variation_id' ] ) : 0;
		$variation = nmgr_get_posted_variations( $variation_id, $_POST );
		$passed_validation = apply_filters( 'woocommerce_add_to_cart_validation', true, $product_id, $quantity );

		if ( $passed_validation && false !== WC()->cart->add_to_cart( $product_id, $quantity, $variation_id, $variation ) ) {
			wc_add_to_cart_message( array( $product_id => $quantity ), true );
		}

		// We are expecting notices from the add to cart action, so get them
		$success = wc_notice_count( 'success' ) ? true : false;
		$notice = wc_print_notices( true );
		wp_send_json( array(
			'success' => $success,
			'notice' => $notice,
		) );
		// phpcs:enable
	}

	public static function get_cart_fragments() {
		if ( class_exists( 'wc_ajax' ) && method_exists( 'wc_ajax', 'get_refreshed_fragments' ) ) {
			WC_AJAX::get_refreshed_fragments();
		}
	}

	public static function send_ajax_response( $data = array() ) {
		if ( !is_ajax() ) {
			return;
		}

		$data = ( array ) $data;

		$error = isset( $data[ 'error' ] ) ? $data[ 'error' ] : (wc_notice_count( 'error' ) ? true : false);
		$success = isset( $data[ 'success' ] ) ? $data[ 'success' ] : (wc_notice_count( 'success' ) ? true : false);

		unset( $data[ 'error' ] );
		unset( $data[ 'success' ] );

		// This clears the notices after printing so we get it last
		$data[ 'notice' ] = isset( $data[ 'notice' ] ) ? $data[ 'notice' ] : wc_print_notices( true );

		wp_send_json( array(
			'error' => $error,
			'success' => $success,
			'data' => $data
		) );
	}

	/**
	 * Save a wishlist profile
	 *
	 * This function is used for both admin and frontend (ajax requests)
	 */
	public static function save_profile() {
		$response_data = array();

		try {
			$posted_form_data = filter_input( INPUT_POST, 'data' );
			$wishlist_id = nmgr_get_verified_wishlist_id();
			$form_data = array();

			if ( $posted_form_data ) {
				parse_str( $posted_form_data, $form_data );
			}

			if ( is_null( $wishlist_id ) || !nmgr_verify_form_nonce( $form_data ) ) {
				throw new Exception( sprintf(
						/* translators: %s: wishlist type title */
						__( 'We could not save your %s details, please try again.', 'nm-gift-registry-lite' ),
						nmgr_get_type_title()
					) );
			}

			// Let's flag if this is a new wishlist.
			if ( 0 === $wishlist_id ) {
				$response_data[ 'created' ] = true;
			}

			$form = new NMGR_Form( $wishlist_id );
			$form->sanitize( $form_data )->validate();

			if ( $form->has_errors() ) {
				foreach ( $form->get_error_messages() as $message ) {
					wc_add_notice( $message, 'error' );
				}
			} else {
				$id = $form->save();

				if ( !$id ) {
					throw new Exception( sprintf(
							/* translators: %s: wishlist type title */
							__( 'Sorry the %s details could not be saved.', 'nm-gift-registry-lite' ),
							nmgr_get_type_title()
						) );
				}

				wc_add_notice(
					/* translators: %s: wishlist type title */
					sprintf( __( 'Your %s details have been saved', 'nm-gift-registry-lite' ),
						nmgr_get_type_title() ),
					'success' );

				$response_data[ 'wishlist' ] = nmgr_get_wishlist( $id )->get_data();
				$response_data[ 'html' ] = nmgr_get_profile_template( $id );
			}
		} catch ( Exception $e ) {
			wc_add_notice( $e->getMessage(), 'error' );
		}

		self::send_ajax_response( $response_data );
	}

	public static function add_item() {
		check_ajax_referer( 'nmgr-items', 'security' );

		try {
			if ( !isset( $_POST[ 'wishlist_id' ] ) ) {
				/* translators: %s: wishlist type title */
				throw new Exception( sprintf( __( 'Invalid %s', 'nm-gift-registry-lite' ), nmgr_get_type_title() ) );
			}

			$wishlist_id = absint( wp_unslash( $_POST[ 'wishlist_id' ] ) );
			$wishlist = nmgr_get_wishlist( $wishlist_id, true );

			if ( !$wishlist ) {
				/* translators: %s: wishlist type title */
				throw new Exception( sprintf( __( 'Invalid %s', 'nm-gift-registry-lite' ), nmgr_get_type_title() ) );
			}

			// If we passed through items it means we need to save first before adding a new one.
			if ( !empty( $_POST[ 'items' ] ) ) {
				$save_items = array();
				parse_str( wp_unslash( $_POST[ 'items' ] ), $save_items );
				$wishlist->update_items( wc_clean( $save_items ) );
			}

			$items_to_add = isset( $_POST[ 'data' ] ) ? array_filter( wc_clean( wp_unslash( ( array ) $_POST[ 'data' ] ) ) ) : array();

			// Add items to wishlist.
			foreach ( $items_to_add as $item ) {
				if ( !isset( $item[ 'id' ], $item[ 'qty' ] ) || empty( $item[ 'id' ] ) ) {
					continue;
				}
				$product_id = absint( $item[ 'id' ] );
				$qty = wc_stock_amount( $item[ 'qty' ] );
				$product = wc_get_product( $product_id );
				$favourite = isset( $item[ 'fav' ] ) ? absint( $item[ 'fav' ] ) : 0;

				if ( !$product ) {
					throw new Exception( __( 'Invalid product ID', 'nm-gift-registry-lite' ) . ' ' . $product_id );
				}

				$wishlist->add_item( $product, $qty, $favourite );
			}

			$items_html = nmgr_get_items_template( $wishlist );
			wp_send_json_success( array( 'html' => $items_html ) );
		} catch ( Exception $e ) {
			wp_send_json_error( array( 'error' => $e->getMessage() ) );
		}
	}

	public static function delete_item() {
		check_ajax_referer( 'nmgr-items', 'security' );
		$response_data = array();

		try {
			$wishlist_id = nmgr_get_verified_wishlist_id();
			$error_msg = __( 'The item(s) could not be deleted', 'nm-gift-registry-lite' );

			if ( !$wishlist_id || !isset( $_POST[ 'wishlist_item_ids' ] ) ) {
				throw new Exception( $error_msg );
			}

			$wishlist = nmgr_get_wishlist( $wishlist_id );

			if ( !$wishlist ) {
				throw new Exception( $error_msg );
			}

			$wishlist_item_ids = array_map( 'sanitize_text_field', wp_unslash( ( array ) $_POST[ 'wishlist_item_ids' ] ) );

			// If we passed through items it means we need to save first before deleting.
			if ( !empty( $_POST[ 'items' ] ) ) {
				$save_items = array();
				parse_str( wp_unslash( $_POST[ 'items' ] ), $save_items ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
				$wishlist->update_items( wc_clean( $save_items ) );
			}

			if ( !empty( $wishlist_item_ids ) ) {
				foreach ( $wishlist_item_ids as $item_id ) {
					$item_id = absint( $item_id );
					$wishlist->delete_item( $item_id );
				}
			}

			$response_data[ 'success' ] = true;
			$response_data[ 'html' ] = nmgr_get_items_template( $wishlist_id );
			$response_data[ 'wishlist' ] = $wishlist->get_data();
		} catch ( Exception $e ) {
			$response_data[ 'error' ] = true;
			$response_data[ 'notice' ] = $e->getMessage();
		}
		self::send_ajax_response( $response_data );
	}

	public static function save_items() {
		check_ajax_referer( 'nmgr-items', 'security' );
		$wishlist_id = nmgr_get_verified_wishlist_id();

		if ( $wishlist_id && isset( $_POST[ 'items' ] ) ) {
			$wishlist = nmgr_get_wishlist( $wishlist_id );

			if ( $wishlist ) {
				// Parse the jQuery serialized items.
				$items = array();
				parse_str( wp_unslash( $_POST[ 'items' ] ), $items ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
				// Update wishlist items
				$wishlist->update_items( wc_clean( $items ) );

				$response_data = array(
					'wishlist' => $wishlist->get_data(),
					'items' => $items,
					'html' => nmgr_get_items_template( $wishlist_id ),
					'success' => true,
				);

				self::send_ajax_response( $response_data );
			}
		}
		wp_die();
	}

	/**
	 * Search for products and echo json.
	 * @since 1.0.5
	 */
	public static function json_search_products() {
		check_ajax_referer( 'nmgr-search-products', 'security' );

		$term = '';
		$include_variations = true;

		if ( empty( $term ) && isset( $_GET[ 'term' ] ) ) {
			$term = ( string ) wc_clean( wp_unslash( $_GET[ 'term' ] ) ); // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		}

		if ( empty( $term ) ) {
			wp_die();
		}

		if ( !empty( $_GET[ 'limit' ] ) ) {
			$limit = absint( $_GET[ 'limit' ] );
		} else {
			$limit = absint( apply_filters( 'nmgr_json_search_limit', 30 ) );
		}

		$include_ids = !empty( $_GET[ 'include' ] ) ? array_map( 'absint', ( array ) wp_unslash( $_GET[ 'include' ] ) ) : array();
		$exclude_ids = !empty( $_GET[ 'exclude' ] ) ? array_map( 'absint', ( array ) wp_unslash( $_GET[ 'exclude' ] ) ) : array();

		$exclude_types = array();
		if ( !empty( $_GET[ 'exclude_type' ] ) ) {
			// Support both comma-delimited and array format inputs.
			$exclude_types = wp_unslash( $_GET[ 'exclude_type' ] );
			if ( !is_array( $exclude_types ) ) {
				$exclude_types = explode( ',', $exclude_types );
			}

			// Sanitize the excluded types against valid product types.
			foreach ( $exclude_types as &$exclude_type ) {
				$exclude_type = strtolower( trim( $exclude_type ) );
			}
			$exclude_types = array_intersect(
				array_merge( array( 'variation' ), array_keys( wc_get_product_types() ) ),
				$exclude_types
			);
		}

		$data_store = WC_Data_Store::load( 'product' );
		$ids = $data_store->search_products( $term, '', ( bool ) $include_variations, false, $limit, $include_ids, $exclude_ids );

		$product_objects = array_filter( array_map( 'wc_get_product', $ids ), 'wc_products_array_filter_readable' );
		$products = array();

		foreach ( $product_objects as $product_object ) {
			if ( in_array( $product_object->get_type(), $exclude_types, true ) ) {
				continue;
			}

			$formatted_name = $product_object->get_formatted_name();
			$managing_stock = $product_object->managing_stock();

			if ( $managing_stock && !empty( $_GET[ 'display_stock' ] ) ) {
				$stock_amount = $product_object->get_stock_quantity();
				$formatted_name .= ' &ndash; ' . sprintf(
						/* translators: %d: stock quantity */
						__( 'Stock: %d', 'nm-gift-registry-lite' ), wc_format_stock_quantity_for_display( $stock_amount, $product_object ) );
			}

			$products[ $product_object->get_id() ] = rawurldecode( $formatted_name );
		}

		wp_send_json( $products );
	}

	public static function load_overview() {
		check_ajax_referer( 'nmgr-overview', 'security' );
		$wishlist_id = filter_input( INPUT_POST, 'wishlist_id', FILTER_VALIDATE_INT );
		nmgr_get_overview_template( $wishlist_id, true );
		wp_die();
	}

	public static function load_profile() {
		check_ajax_referer( 'nmgr-profile', 'security' );
		$wishlist_id = filter_input( INPUT_POST, 'wishlist_id', FILTER_VALIDATE_INT );
		nmgr_get_profile_template( $wishlist_id, true );
		wp_die();
	}

	public static function load_items() {
		check_ajax_referer( 'nmgr-items', 'security' );
		$wishlist_id = filter_input( INPUT_POST, 'wishlist_id', FILTER_VALIDATE_INT );
		nmgr_get_items_template( $wishlist_id, true );
		wp_die();
	}

	public static function load_shipping() {
		check_ajax_referer( 'nmgr-shipping', 'security' );
		$wishlist_id = filter_input( INPUT_POST, 'wishlist_id', FILTER_VALIDATE_INT );
		nmgr_get_shipping_template( $wishlist_id, true );
		wp_die();
	}

	/**
	 * Save a wishlist's shipping address
	 */
	public static function save_shipping() {
		$response_data = array();

		$posted_form_data = filter_input( INPUT_POST, 'data' );
		$wishlist_id = nmgr_get_verified_wishlist_id();
		$form_data = array();

		if ( $posted_form_data ) {
			parse_str( $posted_form_data, $form_data );
		}

		if ( !$wishlist_id || !nmgr_verify_form_nonce( $form_data ) ) {
			wc_add_notice( sprintf(
					/* translators: %s: wishlist type title */
					__( 'We could not save your %s shipping details, please try again.', 'nm-gift-registry-lite' ),
					nmgr_get_type_title()
				), 'error' );
			self::send_ajax_response();
		}

		$form = new NMGR_Form( $wishlist_id );
		$wishlist = $form->get_wishlist();

		// Always save the ship_to_account_address field
		$use_account_shipping = isset( $form_data[ 'nmgr_ship_to_account_address' ] ) ? $form_data[ 'nmgr_ship_to_account_address' ] : null;
		$wishlist->set_ship_to_account_address( sanitize_title( $use_account_shipping ) );
		$wishlist->save();

		if ( $use_account_shipping ) {
			// Reset wishlist shipping values if users wants to ship to account address
			$default_shipping = $wishlist->get_default_data()[ 'shipping' ];

			$shipping = array();
			foreach ( $default_shipping as $key => $value ) {
				$shipping[ "shipping_$key" ] = $value;
			}

			$wishlist->set_props( $shipping );
			$wishlist->save();
			wc_add_notice( __( 'Your shipping details have been saved', 'nm-gift-registry-lite' ), 'success' );
		} else {
			$form->sanitize( $form_data )->validate();

			if ( $form->has_errors() ) {
				foreach ( $form->get_error_messages() as $message ) {
					wc_add_notice( $message, 'error' );
				}
			} else {
				$form->save();
				wc_add_notice( __( 'Your shipping details have been saved.', 'nm-gift-registry-lite' ), 'success' );
			}
		}

		$wishlist_data = array_merge( $wishlist->get_data(), array( 'has_shipping_address' => $wishlist->has_shipping_address() ) );
		$response_data[ 'wishlist' ] = $wishlist_data;
		$response_data[ 'html' ] = nmgr_get_shipping_template( $wishlist_id );

		self::send_ajax_response( $response_data );
	}

	/**
	 * Search for users
	 */
	public static function json_search_users() {
		ob_start();

		check_ajax_referer( 'nmgr-search-users', 'security' );

		if ( !current_user_can( 'edit_' . nmgr()->post_type_plural ) ) {
			wp_die( -1 );
		}

		$term = isset( $_GET[ 'term' ] ) ? ( string ) wc_clean( wp_unslash( $_GET[ 'term' ] ) ) : ''; // phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
		$limit = 0;

		if ( empty( $term ) ) {
			wp_die();
		}

		$ids = array();
		// Search by ID.
		if ( is_numeric( $term ) ) {
			$customer = new WC_Customer( intval( $term ) );

			// Customer exists.
			if ( 0 !== $customer->get_id() ) {
				$ids = array( $customer->get_id() );
			}
		}

		/**
		 * If the customer doesn't exist by searching by id, search for numeric username,
		 * this prevents performance issues with ID lookups.
		 */
		if ( empty( $ids ) ) {
			$data_store = WC_Data_Store::load( 'customer' );

			/**
			 * If search is smaller than 3 characters, limit result set to avoid
			 * too many rows being returned.
			 */
			if ( 3 > strlen( $term ) ) {
				$limit = 20;
			}
			$ids = $data_store->search_customers( $term, $limit );
		}

		$found_customers = array();

		if ( !empty( $_GET[ 'exclude' ] ) ) {
			$ids = array_diff( $ids, array_map( 'absint', ( array ) wp_unslash( $_GET[ 'exclude' ] ) ) );
		}

		foreach ( $ids as $id ) {
			$customer = new WC_Customer( $id );
			$found_customers[ $id ] = sprintf(
				/* translators: $1: customer name, $2: customer email */
				esc_html__( '%1$s (%2$s)', 'nm-gift-registry-lite' ),
				$customer->get_first_name() . ' ' . $customer->get_last_name(),
				$customer->get_email()
			);
		}

		wp_send_json( $found_customers );
	}

}
